/*
 * formula1.c
 * --- CALCOLO CLASSIFICA CAMPIONATO FORMULA 1 ---
 * Il programma legge file contenenti l'odine di arrivo di un gran premio.
 * I nome dei file sono tipo: gara01.txt, gara02.txt, gara03.txt, ...
 * Ogni file contiene un numero di righe pari ai piloti classificati e per
 * ogni riga una stringa tipo "PILOTA SCUDERIA" in ordine di classifica.
 * Vengono elaborati tanti file garaNN.txt quanti ce ne sono nella 
 * directory e si stampa a video la classifica dei piloti e la classifica
 * delle scuderie.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define NUM_PILOTI 22
#define NUM_SCUDERIE 11

/* 
 * Variabili globali per tenere traccia del numero di piloti e scuderie
 * inseriti
 */
int num_piloti_inseriti = 0;
int num_scuderie_inserite = 0;

/* Array per calcolare i punti delle singole gare */
#define POS_A_PUNTI 10
int punti_per_pos[] = {25, 18, 15, 12, 10, 8, 6, 4, 2, 1};


/* Struttura base per contenere sia i piloti che le scuderie */
struct classificato {
	char nome[30];
	int punti;
};


/*
 * Restituisce un puntatore a struct classificato se trova in classifica
 * il nome passato come parametro oppure NULL se il nome non è presente.
 */
struct classificato* get_classificato_by_name( struct classificato *classifica, int size, char *nome )
{
	int i=0;
	for (i=0; i<size; i++) {
		if (!strcmp(classifica[i].nome, nome)) {
			return &classifica[i];
		}
	}
	return NULL;
}

/*
 * Legge un file di gara e modifica le classifiche 
 */
void leggi_filegara ( FILE *fin, struct classificato *piloti, struct classificato *scuderie)
{
	char buffer[100];
	int pos;
	
	char nome_pilota[30];
	char nome_scuderia[30];
	int punti;
	
	for (pos=0; fgets(buffer, sizeof(buffer), fin); pos++){
		sscanf(buffer, "%s %s", nome_pilota, nome_scuderia);
		
		/* Colacolo i punti per la posizione */
		if (pos<POS_A_PUNTI)
			punti = punti_per_pos[pos];
		else
			punti = 0;
			
		/* Aggiungo alla classifica piloti */
		struct classificato *pilota = get_classificato_by_name(piloti, num_piloti_inseriti, nome_pilota);
		if ( pilota ) {
			pilota->punti += punti;
		} else {
			strcpy(piloti[num_piloti_inseriti].nome, nome_pilota);
			piloti[num_piloti_inseriti].punti = punti;
			num_piloti_inseriti++;
		}
		
		/* Aggiungo alla classifica scuderie */
		struct classificato *scuderia = get_classificato_by_name(scuderie, num_scuderie_inserite, nome_scuderia);
		if ( scuderia ) {
			scuderia->punti += punti;
		} else {
			strcpy(scuderie[num_scuderie_inserite].nome, nome_scuderia);
			scuderie[num_scuderie_inserite].punti = punti;
			num_scuderie_inserite++;
		}
		
		/* 
		 * NOTA
		 * Il codice per aggiungere in classifica piloti e in classifica
		 * scuderie e` molto simile. Come si puo` fare per evitare questa
		 * duplicazione di codice?
		 */
	}
}

int main( int argc, char* argv[] )
{
	/* creo in memoria gli array per memorizzare le informazioni */
	struct classificato piloti[NUM_PILOTI];
	struct classificato scuderie[NUM_SCUDERIE];
	
	/* Variabili per gestire i file da leggere */
	char filename[30];
	FILE *fin;
	int num_gare = 1;
	
	/* Lettura dei file */
	do {
		sprintf(filename, "gara%02d.txt", num_gare);
		num_gare++;
		fin = fopen(filename, "r");
		if (fin){
			leggi_filegara(fin, piloti, scuderie);
		}
	} while (fin);
	
	
	/* TODO: ordinare le classifiche */
	
	/* Stampa */
	int i;
	for (i=0; i<num_piloti_inseriti; i++) {
		printf("%s %d\n", piloti[i].nome, piloti[i].punti);
	}
	
	printf("\n");
	for (i=0; i<num_scuderie_inserite; i++) {
		printf("%s %d\n", scuderie[i].nome, scuderie[i].punti);
	}
	
	return 0;
}
