#include<stdio.h>
#include<string.h>
#include<stdlib.h>

/*
 * Questa funzione, per funzionare, ha bisogno di almeno un file chiamato garaXX.txt
 * all'interno della directory corrente, con XX da 00 a 99.
 * è possibile modificare il numero massimo agendo sulla macro N_GARE e sistemando
 * opportuni parametri nella sprintf che fornisce il nome.
 * Per modificare la dimensione predefinita di allocazione della memoria,
 * bisoga agire sulle variabili globali size_piloti e size_scuderie, di default
 * impostate a 1024 (è comunque prevista una riallocazione della memoria nel caso
 * non fose sufficente questo valore).
 * Agendo su DIM si può invece modificare la dimensione massima dei nomi singoli
 * di piloti e scuderie.
 */

#define DIM (100)
#define N_GARE (100)

/* variabili globali */
int size_piloti = 1024, size_scuderie = 1024;
int num_piloti = 0;
int num_scuderie = 0;

/* strutture contenenti nome (pilota/scuderia) e il punteggio associato */
typedef struct pilota {
    char nome[DIM];
    int punteggio;
} PILOTA;

typedef struct scuderia {
    char nome[DIM];
    int punteggio;
} SCUDERIA;


/* prototipi funzione, in tutto sono 6 */
int get_pilota(PILOTA *pilot, char *nome_pilota, int posizione);

int get_scuderia(SCUDERIA *scud, char *nome_scuderia, int posizione);

int to_punteggio(int posizione);

int cmp_punteggio_piloti(const void *puntat1, const void *puntat2);

int cmp_punteggio_scuderie(const void *punt1, const void *punt2);

void ordinamento(PILOTA *pilot, SCUDERIA *scud);

/*
 * INIZIO DEL MAIN
 */

int main(){
    
    /* i è il contatore generico del ciclo for, posizione avrà il duplice scopo di
     * determinare la posizione dei piloti/scuderie finché nel ciclo, e all'uscita
     * restituire un controllo sull'effettiva apertura di almeno un file */

    int i, posizione = 0, count = 0;
    char nome_gara[15] = {0}, temp[2* DIM];
    char nome_pilota[DIM], nome_scuderia[DIM];

    PILOTA *pilot;  //strutture
    SCUDERIA *scud;
    
    FILE *fin;


    /* Allocazioni della memoria per i puntatori a struttura pilot e scud */
    if ( ! (pilot = malloc(size_piloti * sizeof(PILOTA))) ) {
        fprintf(stderr, "Errore nell'allocazione della memoria per i piloti, alla riga %d\n", __LINE__);
        return -1;
    }

    if ( ! (scud = malloc(size_scuderie * sizeof(SCUDERIA))) ) {
        fprintf(stderr, "Errore nell'allocazione della memoria per le scuderie, alla riga %d\n", __LINE__);
        return -2;
    }


    /* Il ciclo cuore del programma:
     * 1) crea il nome del file gara e lo apre;
     * 2) legge riga per riga e assegna alle struct i nomi e i punteggi di piloti e scuderie
     * 3) rialloca la memoria delle se non sufficiente */

    for (i = 0; i < N_GARE; i++){
        sprintf(nome_gara, "gara%.2d.txt", i);

        if ( (fin = fopen(nome_gara, "r")) != NULL){
        
            posizione = 1;
            
            count = 0;

            while ((fgets(temp, sizeof(temp), fin)) != NULL){
                
                count++;
                strcpy(nome_pilota, "");
                strcpy(nome_scuderia, "");
                
                /* restituisce errore se la stringa contiene solo un'andata a capo */
                if (!(strcmp(temp, "\n"))) {
                    fprintf(stderr, "Errore nella lettura della linea %d di %s\n", count, nome_gara);
                    continue;
                }

                sscanf(temp, "%s %s", nome_pilota, nome_scuderia);

                /* restituisce un errore se non è presente il nome del pilota (forse inutile) */
                if (!(strcmp(nome_pilota, ""))) {
                    fprintf(stderr, "Errore nella lettura della linea %d di %s\n", count, nome_gara);
                    continue;
                }
                /* restituisce errore se non è presente il nome della scuderia */
                if (!(strcmp(nome_scuderia, ""))) {
                    fprintf(stderr, "Errore nella lettura della linea %d di %s\n", count, nome_gara);
                    continue;
                }


                /* chiamate a funzioni */
                get_pilota(pilot, nome_pilota, posizione);
                get_scuderia(scud, nome_scuderia, posizione);
                
                posizione++;    //serve per punteggio e controllo su apertura file

                /* riallocazione della memoria */
                if(num_piloti >= size_piloti) {
                    size_piloti *= 2;
                    if(! (pilot = realloc(pilot, size_piloti * sizeof(PILOTA)))){
                        fprintf(stderr, "Errore nella ri-allocazione della memoria per i piloti, alla riga %d\n", __LINE__);
                        return -3;
                    }
                    fprintf(stdout, "Memoria riallocata\n");
                }
                if(num_scuderie >= size_scuderie) {
                    size_scuderie *= 2;
                    if ( !(scud = realloc(scud, size_scuderie * sizeof(SCUDERIA)))){
                        fprintf(stderr, "Errore nella ri-allocazione della memoria per le scuderie, alla riga %d\n", __LINE__);
                        return -4;
                    }
                    fprintf(stdout, "Memoria riallocata\n");
                }
            }


        fclose(fin);
        }
    }
    
    /*
     * Eseguo un controllo su "posizione". Se è ancora = 0, non sono mai riuscito ad aprire nemmeno un file.
     */

    if (!(posizione)){
        fprintf(stderr, "Errore nell'apertura dei file: file non trovati\n");
        return -5;
    }

    /*effettua la stampa già ordinata */

    ordinamento(pilot, scud);
    

    free(pilot);
    free(scud);
    
    return 0;
}


/* Si occupa di controllare se il pilota esiste già, di crearlo se necessario,
 * e di assegnargli un punteggio */

int get_pilota(PILOTA *pilot, char *nome_pilota, int posizione){
    
    int i, j, a;
    a = 0;
    
    /* 
     * Effettua un controllo entro i limiti del puntatore a struttura pilot,
     * che contiene i nomi dei piloti e i punteggi, e, nome per nome, controlla
     * se il pilota è già stilato, e se lo è, incrementa il punteggio. Se il ciclo
     * ha successo, previene l'esecuzione del successivo (che creerebbe un altro
     * pilota identico), esce dalla funzione ritornando 0
     */
    for ( i = 0; i <= size_piloti; i++){
        
        if(!(strcmp(nome_pilota, pilot[i].nome))){
            
            pilot[i].punteggio += to_punteggio(posizione);  //chiama la funzione to_punteggio
            return 0;
        }
    }

    /*
     * Se il ciclo precedente non ha trovato il pilota, lo crea, controllando
     * quale è il primo elemento "vuoto" di pilot che ancora non contiene un nome.
     * Inoltre assegna il punteggio corrispondente, e assegna un valore a num_piloti,
     * poiché il numero dei nomi sarà il numero dell'ultimo scritto + 1.
     * Esce e ritorna 0.
     */
    for(j=0; (j <= size_piloti) && (a == 0) ; j++){
        if ( !(strcmp(pilot[j].nome, ""))) {
            strcpy(pilot[j].nome, nome_pilota);
            pilot[j].punteggio += to_punteggio(posizione);  //chiama to_punteggio
            num_piloti = j + 1;
            return 0;
        }
    }


    return 1;
}

/* Assolutamente analoga a get_pilot */
int get_scuderia(SCUDERIA *scud, char *nome_scuderia, int posizione){
    
    int i, j, a;
    a = 0;

    for ( i = 0; (i <= size_scuderie); i++){        //controllo esteso a tutti gli elementi
        
        if(!(strcmp(scud[i].nome, nome_scuderia))){ //se trova il pilota, assegna il punteggio ed esce dalla funzione
            
            scud[i].punteggio += to_punteggio(posizione);
            return 0;
        }
    }

    for(j=0; (j <= size_scuderie) && (a == 0); j++){
        if ( !(strcmp(scud[j].nome, ""))) {     //cerca il primo "slot" vuoto, crea il pilota e assegna il punteggio
            strcpy(scud[j].nome, nome_scuderia);
            scud[j].punteggio += to_punteggio(posizione);
            num_scuderie = j+1;
            return 0;       
        }
    }


    return 1;
}


/* Funzione che ritorna il punteggio equivalente alla posizione in graduatoria, 0 da 10 in poi */
int to_punteggio(int posizione){

    int punteggio = 0;

    if (posizione == 1)
        punteggio = 25;
    else if (posizione == 2)
        punteggio = 18;
    else if (posizione == 3)
        punteggio = 14;
    else if (posizione == 4)
        punteggio = 12;
    else if (posizione == 5)
        punteggio = 10;
    else if (posizione == 6)
        punteggio = 8;
    else if (posizione == 7)
        punteggio = 6;
    else if (posizione == 8)
        punteggio = 4;
    else if (posizione == 9)
        punteggio = 2;
    else if (posizione == 10)
        punteggio = 1;
    else punteggio = 0;

    return punteggio;
}

/*
 * Funzione comparativa, serve alla qsort. Richiede i puntatori ad un generico void,
 * che devono poi essere assegnati alla giusta struttura.
 */
int cmp_punteggio_scuderie(const void *punt1, const void *punt2) {
    
    /* 
     * Prendo punt1 e punt2, puntatori a void richiesti per la qsort, e li riassegno al tipo
     * di struttura pertinente in modo che puntino agli elementi del vettore da ordinare.
     */

    SCUDERIA *scud1 = (SCUDERIA *)(punt1),
             *scud2 = (SCUDERIA *)(punt2);

    /* riassegna l'intera "cella" di struct sulla base della dimensione del punteggio,
     * se il primo elemento puntato è più grande, esso andrà prima rispetto al secondo */
    return (scud2->punteggio - scud1->punteggio);
}


/* Funzione comparativa analoga a cmp_punteggio_piloti */
int cmp_punteggio_piloti(const void *puntat1, const void *puntat2) {

	PILOTA *pilot1 = (PILOTA *)(puntat1),
	       *pilot2 = (PILOTA *)(puntat2);

	return (pilot2->punteggio - pilot1->punteggio);
}


/*
 * Funzione che si occupa di richiamare la qsort e anche di stampare direttamente a video
 * il contenuto delle strutture una volta riordinato.
 */
void ordinamento(PILOTA *pilot, SCUDERIA *scud) {

	int i;

    /* qsort chiama la funzione cmp_punteggio_... e riordina gli elementi del puntatore alle struct */

	qsort(pilot, num_piloti, sizeof(PILOTA), cmp_punteggio_piloti);
    qsort(scud, num_scuderie, sizeof(SCUDERIA), cmp_punteggio_scuderie);

    printf("\n");
    puts("Classifica dei piloti:");
	for (i = 0; i < num_piloti; i++)
		printf("%s %2d\n", pilot[i].nome, pilot[i].punteggio);

    printf("\n");
    puts("Classifica delle scuderie:");
    for (i = 0; i < num_scuderie; i++)
        printf("%s %2d\n", scud[i].nome, scud[i].punteggio);

}
