/*Fondamenti di Informatica		23/01/2013

Problema
Il file materiali.dat elenca su ciascuna riga il nome di un materiale e il suo peso specifico in kg/m3 . Per esempio:

3
Argento 10500
Granito 2500
Ossigeno 1.429

La prima riga del file contiente un unico numero intero che indica il numero di materiali successivamente elencati. Il nome é una singola
stringa di lunghezza massima pari a 40 caratteri. Il peso specifico é rappresentato come numero in virgola mobile. 
I due valori sono separati da uno spazio.
Il file elenco.dat elenca su ciascuna riga il nome di uno dei materiali presenti in materiali.dat e un volume espresso in dm3 (litri). 
Il volume é rappresentato come numero in virgola mobile. I due valori sono separati da uno spazio. 
La prima riga del file contiene un numero intero che indica il numero di materiali successivamente elencati.

Richiesta
Scrivere un programma che soddisfi i seguenti requisiti:
1. includa due funzioni per caricare i dati dai file nel formato specificato (una per file);
2. comprenda una funzione che calcoli il peso totale dell’insieme di materiali specificato in un file elencoX.dat
   a partire dalle loro quantità e dai pesi specifici riportati in materiali.dat (fare attenzione all’equivalenza tra dm3 e m3 );
3. implementi una funzione che elenchi soltanto i materiali presenti in materiali.dat che iniziano con una
   lettera specificata come parametro (senza considerare la maiuscola/minuscola, ovvero se si specifica la lettera
   ’a’ devono essere elencati i materiali che iniziano sia con ’a’ e che con ’A’);
4. preveda una funzione che ordini in senso crescente di peso specifico l’elenco dei materiali presenti in materiali.dat,
   e che visualizzi l’elenco ordinato; */
   
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<ctype.h>    // serve per la tolower

#define TRACE printf("DEBUG %s: %d\n", __func__, __LINE__);    // we love debugging

typedef struct                   // per materiali.dat
{
	char nome[40];
	float ps;
} MATERIALE;

typedef struct                  // per elencoX.dat
{
	char nome[40];
	float vol;
} ELENCO;

int num_mat=0;                // il numero di materiali in materiali.dat
int num_el=0;                 // il numero di elementi in elencoX.dat


/* La funzione apre il file 'materiali.dat', acquisisce il numero di materiali e poi provvede a riempire la struct preposta.
   Numerosi controlli permettono di intercettare eventuali errore nell'apertura del file o nel formato dello stesso.
   In questo caso, il programma torna nel main per poi terminare con un errore.*/  
MATERIALE *carica_materiali()
{
	MATERIALE *v;
	FILE *f;
	char linea[100];
	int test=0, i;
	
	if(!(f = fopen("materiali.dat", "r")))
		return NULL;
	
	fgets(linea, sizeof(linea), f);
	test = sscanf(linea, "%d", &num_mat);
	if(test != 1)                 // significa che non è presente il numero dei materiali
		return NULL;
		
	if(!(v = malloc(num_mat * sizeof(MATERIALE))))
		return NULL;
	
	for(i=0; i<num_mat; i++)            // riempio la struct
	{
		fgets(linea, sizeof(linea), f);
		if((sscanf(linea, "%s %f", (v+i)->nome, &((v+i)->ps))) != 2)
			return NULL;
	}
		
	return v;   // se sono qui, tutto è andato bene
}


/* Come sopra */
ELENCO *carica_elenco(FILE *f)
{
	ELENCO *v;
	char linea[100];
	int test=0, i;
	
	fgets(linea, sizeof(linea), f);
	test = sscanf(linea, "%d", &num_el);
	if(test != 1)
		return NULL;
		
	if(!(v = malloc(num_el * sizeof(ELENCO))))
		return NULL;
	
	for(i=0; i<num_el; i++)
	{
		fgets(linea, sizeof(linea), f);
		if((sscanf(linea, "%s %f", (v+i)->nome, &((v+i)->vol))) != 2)
			return NULL;
	}
		
	return v;
}


/* La funzione ritorna il peso totale di un elenco.
   Confronta ogni singolo elemento dell'elenco con i materiali acquisiti dal file 'materiali.dat' fintanto che non trova una
   corrispondenza. A quel punto si applica la formula 'm = dv', dove v viene convertito da dm^3 in m^3 (*1/1000).
   Il ciclo continua ad aggiornare il peso tatole fino alla fine dell'elenco.*/
float peso_totale(MATERIALE *m, ELENCO *e)
{
	int i, j;
	float peso=0;
	
	for(i=0; i<num_el; i++)
	{
		for(j=0; j<num_mat; j++)
		{
			if(!(strcmp((e+i)->nome, (m+j)->nome))) 
				peso += (((e+i)->vol)/1000) * ((m+j)->ps);	
		}
	}
	return peso;
}


/* La funzione confronta la chiave di ricerca eventualmente cfornita dall'utente con il primo carattere del nome di ogni materiale.
   L'utilizzo della tolower permette di evitare problemi di case sensitive. Vengono poi stampate le corrispondenze.*/ 
void ricerca_materiali(MATERIALE *p, char c)
{
	int i;
	
	printf("Elenco materiali richiesti:\n");
	for(i=0; i<num_mat; i++)
	{
		if(tolower((p+i)->nome[0]) == tolower(c))
			printf("%d. %s.\n", i+1, (p+i)->nome);
	}
}


/* Funzione necessaria alla qsort */
int cmp_ps(const void *p1, const void *p2)
{
	MATERIALE *ptr1 = (MATERIALE *)p1, *ptr2 = (MATERIALE *)p2;
	return ptr1->ps - ptr2->ps;
} 


/* La funzione stampa a ciclo tutte le informazioni richieste */
void stampa_tutto(MATERIALE *p, float peso)
{
	int i;
	
	printf("\nElenco ordinato:\n");
	for(i=0; i<num_mat; i++)
		printf("%2d-> %s %f\n", i+1, (p+i)->nome, (p+i)->ps);
	printf("\nPeso totale dell'elenco: %f kg.\n\n", peso);
}	 


int main(int argc, char *argv[])
{
	FILE *fin;        // per l'elenco
	MATERIALE *p1;    // puntatori alle 2 struct
	ELENCO *p2;
	float peso;       // il peso totale di un elenco
	char key;         // la chiave di ricerca passata dall'utente 
	
	if(argc < 2 || argc > 3)            // senza almeno un parametro non si può aprire un file di input
	{
		printf("UTILIZZO: ./esame5 elencoX.dat [char]\n");
		return -1;
	}
	
	if(!(fin = fopen(argv[1], "r")))
	{
		printf("ERRORE nell'apertura del file di input.\n");
		return -2;
	}
	
	p1 = carica_materiali();   // ritornano NULL se falliscon per qualche motivo
	p2 = carica_elenco(fin);  

	if(p1 == NULL || p2 == NULL)
	{
		printf("ERRORE nel caricamento dei dati.\n");
		return -3;
	}
	
	peso = peso_totale(p1, p2);    // calcolo il peso
	
	if (argc == 3)                // la ricerca è a discrezione dell'utente
	{	
		if((sscanf(argv[2], "%c", &key)) == 1)		// se viene inserito qualcosa di diverso da un char
			ricerca_materiali(p1, key);
		else
		{
			printf("UTILIZZO: ./esame5 elencoX.dat [char]\n");
			return -4;
		}
	}
	
	qsort(p1, num_mat, sizeof(MATERIALE), cmp_ps);          // ordino...
	
	stampa_tutto(p1, peso);                                // e stampo
	
	return 0;
}

/* Per ottenere l'elaborazione di un file diverso da 'materiali.dat' è sufficiente modificare il parametro della fopen
   nella funzione carica_materiali.
   Se invece si volesse ottenere la stampa su file si può utilizzare la redirezione dell'output.
*/


/* NO FATE BUT WHAT WE MADE */




