/* Fondamenti di Informatica     01/03/2013

Sia dato un file che riporta i risultati della valutazione di un sistema di riconoscimento di persone all’interno di
immagini. Il sistema è stato in grado di classificare un’immagine in base alla persona ritratta. Le persone che hanno
partecipato alla sperimentazione sono Anna, Agnese, Elvira, Sara, Giovanna, Viola, Alba. Il file riporta nella prima
riga il numero di immagini utilizzate per il test e su ciascuna riga successiva le seguenti informazioni separate da
spazio:
• il codice identificativo dell’immagine analizzata (numero intero);
• due numeri interi che indicano la dimensione dell’immagine in pixel (larghezza per altezza);
• il nome della persona presente nell’immagine (stringa);
• il nome della persona riconosciuta (stringa), oppure la stringa “ERRORE” in caso di problemi di riconoscimento.

Un esempio del contenuto del file è il seguente:

3
1 800 600 Anna Anna
2 1024 768 Elvira Agnese
3 640 480 Sara ERRORE

Si sviluppi un programma che, letto il file, soddisfi le seguenti richieste:
  1. stampi a video il numero di riconoscimenti errati (viene riportata la stringa “ERRORE”) e la percentuale sul
totale di immagini;
  2. stampi a video il numero totale di immagini correttamente classificate dal sistema indipendentemente dalla
persona presente (persona presente = persona riconosciuta), e la loro percentuale rispetto al totale di immagini
riconosciute senza errore;
  3. scartando i riconoscimenti nei quali viene segnalato “ERRORE”, stampare a video, per ogni insieme di immag-
ini che rappresentano una persona, il numero e la percentuale rispetto al totale di immagini senza “ERRORE”,
e il numero e la percentuale di volte in cui la persona è stata correttamente riconosciuta in esse;
  4. ordinare tutte le righe lette in ordine crescente di area dell’immagine (numero di pixel) e stampare l’insieme
ordinato nel file ord.dat.

Esempio di output a video:
85 riconoscimenti errati su 350 (24.285715%)
183 immagini riconosciute correttamente su 265 (69.056610%)
Anna: 30 immagini su 265 (11.320755%), 20 riconosciute correttamente (66.666672%)
Agnese: 34 immagini su 265 (12.830189%), 24 riconosciute correttamente (70.588234%)
Elvira: 45 immagini su 265 (16.981133%), 33 riconosciute correttamente (73.333336%)
...
NOTA: se ne si è in grado, impostare il programma in modo da poterlo estendere facilmente per gestire molte più
persone coinvolte nell’esperimento di riconoscimento utilizzando strutture dati e costrutti di controllo adatti.
*/


/* INIZIO PROGRAMMA */

#include<stdio.h>
#include<stdlib.h>
#include<string.h>

#define TRACE printf("DEBUG %s: %d\n", __func__, __LINE__);

typedef struct
{
	int id;
	int b;
	int h;
	char presente[50];
	char riconosciuta[50];
} TEST;	
char *nomi[]={"Anna", "Agnese", "Elvira", "Sara", "Giovanna", "Viola", "Alba"};
int num_imm;
int num_nomi=sizeof(nomi)/sizeof(char *);


TEST *carica_struct(char *infile)
{
	TEST *v;
	FILE *fin;
	int i, id, base, altezza, check;
	char linea[200], presente[50], riconosciuta[50];
	
	if(!(fin = fopen(infile, "r")))
		return NULL;
	
	fgets(linea, sizeof(linea), fin);
	if(sscanf(linea, "%d", &num_imm) != 1)
		return NULL;
	
	if(!(v = malloc(num_imm * sizeof(TEST))))
		return NULL;
	for(i=0; i<num_imm; i++)
	{
		fgets(linea, sizeof(linea), fin);
		check = sscanf(linea, "%d %d %d %s %s", &id, &base, &altezza, presente, riconosciuta);
		if (check != 5)
			return NULL;
		(v+i)->id = id;
		(v+i)->b = base;
		(v+i)->h = altezza;
		strncpy((v+i)->presente, presente, sizeof((v+i)->presente));
		strncpy((v+i)->riconosciuta, riconosciuta, sizeof((v+i)->riconosciuta));
	}
	fclose(fin);
	return v;
}


int stampa_errati(TEST *v)
{
	int i, count=0;
	float perc;
	
	for (i=0; i<num_imm; i++)
	{
		if(!(strcmp((v+i)->riconosciuta, "ERRORE")))
			count++;
	}
	perc = ((float) count)/num_imm *100;
	printf("%d riconoscimenti errati su %d (%f%%)\n", count, num_imm, perc);
	return count;
}	
	

void stampa_corretti(TEST *v, int errate)
{
	int i, count=0;
	float perc;
	 for (i=0; i<num_imm; i++)
	 {
	 	if(!(strcmp((v+i)->presente, (v+i)->riconosciuta)))
	 		count++;
	 }
	 perc = (((float) count)/(num_imm - errate)) *100;
	 printf("%d immagini riconosciute correttamente su %d (%f%%)\n\n", count, num_imm - errate, perc);
}


void stampa_percentuali(TEST *v, int errate)
{
	int i, j, count1=0, count2=0;
	float perc1, perc2;

	for (i=0; i<num_nomi; i++)
	{
		for(j=0; j<num_imm; j++)
		{
			if((!(strcmp(nomi[i], (v+j)->presente))) && strcmp("ERRORE", (v+j)->riconosciuta))
			{	
				count1++;
				if(!(strcmp((v+j)->riconosciuta, (v+j)->presente)))
				count2++;
			}
		}
		perc1 = ((float)count1/(num_imm-errate))*100;
		perc2 = ((float)count2/count1)*100;
		printf("%s: %d immagini su %d (%f%%), %d riconosciute (%f%%)\n", nomi[i], count1, num_imm - errate, perc1, count2, perc2 );
		count1=0;
		count2=0;
		j=0;
	}
	printf("\n");
}		
				

int cmp_aree(const void *p1, const void *p2)
{
	TEST *v1=(TEST *)p1, *v2=(TEST *)p2;
	return (v1->b)*(v1->h) - (v2->b)*(v2->h);
}
	 

void stampa_ordinata(TEST *v, char *outfile)
{
	FILE *fout;
	int i;
	
	if(!(fout=fopen(outfile, "w")))
	{
		printf("ERRORE: impossibile aprire il file di output.\n");
		exit(-2);
	}
	fprintf(fout, "Riconoscimenti ordinati per area dell'immagine:\n");
	for(i=0; i<num_imm; i++)
		fprintf(fout, "%d ---> %d %d %d %s %s\n", i+1, (v+i)->id, (v+i)->b, (v+i)->h, (v+i)->presente, (v+i)->riconosciuta);
	fclose(fout);
}
			

int main(int argc, char *argv[])
{
	TEST *p;
	char infile[100]="riconoscimenti.dat", outfile[100]="ord.dat";
	int errati;
	
	switch (argc)
	{
		case 1: 
			printf("\n****ATTENZIONE: saranno usati i file di default.****\nSe desiderato specificare i parametri\n\n");
			break;
		case 2:
			printf("\n****ATTENZIONE: inserito solo file di input.****\nSe desiderato specificare l'altro parametro\n\n");
			strncpy(infile, argv[1], sizeof(infile));
			break;
		case 3:
			strncpy(infile, argv[1], sizeof(infile));
			strncpy(outfile, argv[2], sizeof(outfile));
			break;
	}
	
	p = carica_struct(infile);
	if(p == NULL)
	{
		printf("ERRORE nella lettura del file o nell'allocazione della memoria.\n");
		return -1;
	}
	
	errati = stampa_errati(p);
	stampa_corretti(p, errati);
	stampa_percentuali(p, errati);
	qsort(p, num_imm, sizeof(TEST), cmp_aree);
	stampa_ordinata(p, outfile);
	
	free(p);
	
	return 0;
} 
