/* Fondamenti di Informatica                      15/03/2012

Problema
Un file contiene l’estratto dello storico di un sistema che gestisce le transazioni finanziarie di una azienda. Il file
contiene un record (le informazioni su una transazione) per riga. Il formato di ciascuna riga è il seguente:
AAAA MM GG ammontare
dove:
• AAAA è l’anno della transazione (4 caratteri numerici interi); gli anni si riferiscono al periodo [1990, 2005];
• MM è il mese della transazione (2 caratteri numerici interi), rappresentato nel range [1, 12];
• GG è il giorno della transazione (2 caratteri numerici interi); il primo giorno del mese è rappresentato da 1;
• ammontare è l’ammontare della transazione; è un numero in virgola mobile con segno positivo per le entrate
  e negativo per le uscite.
Possono essere memorizzate più transazioni diverse effettuate nella stessa giornata.
Richiesta
Creare un programma che legga il file nel formato specificato e calcoli, stampando a video il risultato, i seguenti
valori:
1. l’anno nel quale ci sono state le maggiori entrate;
2. il mese nelle quali ci sono state le maggiori uscite (per ciascun mese vanno sommati i valori di tutti gli anni);
3. gli anni nei quali si sono avuti il migliore e il peggiore saldo (differenza tra entrate e uscite);
4. la media delle entrate e delle uscite mensili in tutto il periodo considerato; l’output deve essere fornito nel
   formato AAAA MM media, una voce per riga.
Inoltre, ordinare i record per valore crescente dell’ammontare, e memorizzare l’insieme di record ordinati in un file.*/

#include<stdio.h>
#include<stdlib.h>
#include<string.h>

#define TRACE printf("%s: riga %d\n", __func__, __LINE__);
#define AMAX (2005)
#define AMIN (1990)
#define ANNI (AMAX-AMIN+1)

typedef struct
{
	int a;
	int m;
	int g;
	float cash;
} TRANS;

typedef struct
{
	int num_in, num_out;
	float amm_in, amm_out;
} SOMMA;

int num_trans=0;


TRANS *carica_struct(FILE *f)
{
	TRANS *v;
	char linea[512];
	int dim=40, i=0;
	
	if(!(v=malloc(dim*sizeof(TRANS))))
		return NULL;
	
	while((fgets(linea, sizeof(linea), f) != NULL))
	{
		if((sscanf(linea, "%d %d %d %f", &((v+i)->a), &((v+i)->m), &((v+i)->g), &((v+i)->cash))) != 4)
		{
			printf("ERRORE nel formato del file di input, riga %d", num_trans+1);
			return NULL;
		}
		num_trans++;
		if(num_trans >= dim)
		{
			dim *= 2;
			if(!(v=realloc(v, dim*sizeof(TRANS))))
				return NULL;
		}
		i++;
	}
	
	return v;
}
	
	
void a_max_entrate(TRANS *v)
{
	int i, imax=0;
	float max=0;
	float entrate[ANNI]={0};
	
	for(i=0; i<num_trans; i++)
	{
		if((v+i)->cash > 0)
				entrate[((v+i)->a) - AMIN] += ((v+i)->cash);
	}
	
	for(i=0; i<(ANNI); i++)
	{
		if(entrate[i] > max)
		{
			max = entrate[i];
			imax = i;
		}
	}
	printf("L'anno con le maggiori entrate è: %d (%.2f $)\n", AMIN + imax, entrate[imax]);
}	


void m_max_uscite(TRANS *v)
{
	int i, imax=0;
	float max=0;
	float uscite[12]={0};
	
	for(i=0; i<num_trans; i++)
	{
		if((v+i)->cash < 0)
				uscite[(v+i)->m-1] += ((v+i)->cash);
	}
	
	for(i=0; i<(12); i++)
	{
		if(uscite[i] < max)
		{
			max = uscite[i];
			imax = i;
		}
	}
	printf("Il mese con le maggiori uscite è: %d (%.2f $)\n", imax+1, -uscite[imax]);
}	


void a_saldi_estremi(TRANS *v)
{
	int i, imax=0, imin=0;
	float max, min;
	float saldo[ANNI]={0};
	
	for(i=0; i<num_trans; i++)
		saldo[(v+i)->a - AMIN] += ((v+i)->cash);
		
	max = saldo[0];
	for(i=0; i<ANNI; i++)
	{
		if(saldo[i] > max)
		{
			max = saldo[i];
			imax = i;
		}
	}
	printf("L'anno con il miglior saldo è: %d (%.2f $)\n", AMIN + imax, saldo[imax]);
	
	min = saldo[0];
	for(i=0; i<ANNI; i++)
	{
		if(saldo[i] < min)
		{
			min = saldo[i];
			imin = i;
		}
	}
	printf("L'anno con il peggior saldo è: %d (%.2f $)\n\n", AMIN + imin, saldo[imin]);
}	


void in_out(TRANS *v)
{
	SOMMA tab[ANNI][12] = {{{0, 0 , 0, 0}}};
	int i, j;
	
	for(i=0; i<num_trans; i++)
	{
		if((v+i)->cash > 0)
		{
			tab[(v+i)->a - AMIN][(v+i)->m - 1].num_in++;
			tab[(v+i)->a - AMIN][(v+i)->m - 1].amm_in += (v+i)->cash;
		}
		if((v+i)->cash < 0)
		{
			tab[(v+i)->a - AMIN][(v+i)->m - 1].num_out++;
			tab[(v+i)->a - AMIN][(v+i)->m - 1].amm_out += (v+i)->cash;
		}
	}
	
	for (i=0; i<ANNI; i++)
	{	
		for(j=0; j<12; j++)
		{
			if(tab[i][j].num_in > 0)
				printf("%02d/%4d media entrate: %f $\n", j+1, i+AMIN, tab[i][j].amm_in/tab[i][j].num_in);
			if(tab[i][j].num_out > 0)
				printf("%02d/%4d media uscite: %f $\n", j+1, i+AMIN, -tab[i][j].amm_out/tab[i][j].num_out);	
		}
	}
	
}


void stampa_file(TRANS *v, FILE *f)
{
	int i;
	
	fprintf(f, "AAAA MM GG  CASH\n");
	for(i=0; i<num_trans; i++)
		fprintf(f, "%d %02d %02d  %f\n", (v+i)->a, (v+i)->m, (v+i)->g, (v+i)->cash);
}
		

int cmp_cash(const void *c1p, const void *c2p)
{
	TRANS *c1=(TRANS *)c1p, *c2=(TRANS *)c2p;
	return (c1->cash) - (c2->cash); 
}


int main(int argc, char *argv[])
{
	FILE *fin, *fout;
	TRANS *p;
	
	if(argc != 3)
	{
		printf("UTILIZZO: ./esame7 infile outfile\n");
		return -1;
	}
	
	if(!((fin=fopen(argv[1], "r")) && (fout=fopen(argv[2], "w"))))
	{
		printf("ERRORE nell'apertura dei file\n");
		return -2;
	}
	
	p = carica_struct(fin);
	if(p == NULL)
	{
		printf("ERRORE nel caricamento della struttura\n");
		return -3;
	}
	
	a_max_entrate(p);
	m_max_uscite(p);
	a_saldi_estremi(p);
	in_out(p);
	
	qsort(p, num_trans, sizeof(TRANS), cmp_cash);
	stampa_file(p, fout);
	
	fclose(fin);
	fclose(fout);
	free(p);
	
	return 0;
}
