/*Fondamenti di Informatica             26/01/2012

In due file di testo input1.dat e input2.dat, aventi la medesima struttura, sono memorizzati
degli insiemi di vettori. I vettori sono composti da numeri interi con segno. I file sono strutturati
come segue:
•la prima riga contiene il numero di vettori presenti nel file;
•nelle righe seguenti, ciascun vettore e` memorizzato come segue:
	◦ un primo numero, in una riga a se` stante, indica quanti sono i valori che
	compongono il vettore;
	◦ seguono gli elementi del vettore, uno per ciascuna riga.
Ogni riga del file contiene un solo valore numerico.

Si richiede di leggere i file nel formato specificato e di operare le seguenti elaborazioni:
 1. calcolare e stampare su file il valore massimo e minimo per ciascun vettore;
 2. ordinare in senso crescente gli elementi di ciascun vettore; stampare su file i valori
    ordinati, un vettore per riga;
 3. calcolare la somma degli elementi di ciascun vettore e stampare su file il massimo tra le
    somme calcolate, indicando quale e` il vettore che ha prodotto la somma massima (i
    vettori si enumerano partendo da zero).

I risultati elaborati per ciascun file input1.dat e input2.dat devono essere scritti
rispettivamente su out1.dat e out2.dat.

Sapendo che i vettori in posizioni corrispondenti nei due file hanno la stessa dimensione
(uguale numero di elementi), stampare a video il valore del prodotto scalare calcolato per i
vettori di pari posizione. Si stampi un risultato per riga.

Struttura di un file di output corrispondente ad un file di input contenente 2 vettori A e B:
minA maxA
A1 A2 A3 A4 // elementi ordinati del vettore A 
minB maxB
B1 B2 B3 B4 B5 B6 // elementi ordinati del vettore B 
somma_max vett_max
*/



#include<stdio.h>
#include<stdlib.h>

#define TRACE printf("DEBUG: %s, %d\n", __func__, __LINE__);


typedef struct 
{
	int n;                // il contatore di elementi
	int *el;              // un puntatore all'elemento
} VETT;

int num[2]={0};               // il numero di vettori in ciascun file

/* La funzione riceve un puntatore a file. Da questo viene letta la prima riga: gazie all'informazione in essa contenuta,
   è possibile usare la malloc senza prievedere riallocazioni. Poi occorre caricare gli elementi del singolo vettore. Viene
   dapprima letta la dimensione, successivamente un ciclo preleva le entrate e le immagazzina in un'area di memoria allocata 
   ad hoc. Numerosi controlli permetono di intercettare incoerenze dei dati che porterebbero al fallimento delle successive elaborazioni.
   Il numero passato alla funzione serve per verificare che nei file sia effettivamente contenuto lo stesso numero di vettori.
*/  
VETT *carica_struct(FILE *f, int j)
{
	VETT *v;
	int i, k;
	char linea[50];
	
	fgets(linea, sizeof(linea), f);
	if(sscanf(linea, "%d", &num[j]) != 1)
		return NULL;
		
	if(!(v = malloc(num[j] * sizeof(VETT))))
		return NULL;
	
	for(i=0; i<num[j]; i++)
	{
		fgets(linea, sizeof(linea), f);
		if(sscanf(linea, "%d", &((v+i)->n)) != 1)
			return NULL;
		if(!((v+i)->el = malloc(((v+i)->n) * sizeof(int))))
			return NULL;
		for(k=0; k<(v+i)->n; k++)
		{
			fgets(linea, sizeof(linea), f);
			if(sscanf(linea, "%d", &((v+i)->el[k])) != 1)
				return NULL;
		}
	}
	
	return v;
}

/* Funzione di comparazione 'classica' usata da qsort */
int cmp_vett(const void *p1p, const void *p2p)
{
	int *p1=(int *)p1p, *p2=(int *)p2p;
	return *p1 - *p2;
}

/* La funzione assolve il duplice compito di riordinare gli elementi del vettore ed identificare massimo e minimo.
   Ovviamente questa seconda operazione precede la prima, in modo da rendere inutile l'utilizzo di qualsiasi algoritmo:
   la qsort avrà già provveduto a disporre in prima posizione il minimo ed in ultima il massimo.
*/
void stampa_estremi(VETT *v, FILE *f)
{
	int i, j;
	
	for(i=0; i<num[0]; i++)
	{
		qsort((v+i)->el, (v+i)->n, sizeof(int), cmp_vett);
		fprintf(f, "max->%3d  min->%3d\n", (v+i)->el[(v+i)->n - 1], (v+i)->el[0]);
		fprintf(f, "Elementi ordinati: ");
		for(j=0; j<(v+i)->n; j++)
			fprintf(f, "%3d ", (v+i)->el[j]);
		fprintf(f, "\n");
	}
}
	
/* La funzione esegue la somma di tutti gli elementi di un vettore, usando la doppia indicizzazione della struttura
   che contiene il vettore d'interesse e dell'elemento all'interno del vettore. I risultati vengono conservati in un vettore:
   successivamente è sufficiente identificare il massimo elemento del vettore 'somma' per assolvere la richiesta.
   Il vettore somma dev'essere allocato dinamicamente perchè la sua dimensione non è nota a priori.
*/
void stampa_somme(VETT *v, FILE *f)
{
	int i, j, *somma, imax;
	
	if(!(somma=malloc(num[0] * sizeof(int))))
		exit(-4);
		
	for(i=0; i<num[0]; i++)
	{	
		for(j=0; j<(v+i)->n; j++)
			somma[i] += (v+i)->el[j];
	}
		
	imax=0;
	for(i=0; i<num[0]; i++)
	{
		if(somma[i] > somma[imax])
			imax = i;	
	}
	fprintf(f, "Somma max->%4d     Indice max->%2d\n", somma[imax], imax);
}					
	
/* La funzione, previo controllo dei necessari requisiti (pari numero di elementi di ciascun vettore), esegue il prodotto 
   scalare applicando la definizione matematica.
*/	
void scalare(VETT *v1, VETT *v2)
{
	int i, j, scalare;
	
	printf("Prodotti scalari:\n");
	for(i=0; i<num[0]; i++)
	{
		if((v1+i)->n != (v2+i)->n)
		{
			printf("Dati non coerenti per il prodotto scalare: vettore %d\n", i+1);
			exit(-6);
		}	
		scalare=0;
		for(j=0; j<(v1+i)->n; j++)
			scalare += ((v1+i)->el[j]) * ((v2+i)->el[j]);
		printf("%d ----> %d\n", i+1, scalare);
	}
	printf("\n");
}


int main(int argc, char *argv[])
{
	FILE *fin1, *fin2, *fout1, *fout2;
	VETT *p1, *p2;
	
	switch (argc)
	{
		case 1:   // non si è specificato alcun parametro: il programma tenta di usare i file di default
		{
			printf("\nATTENZIONE: verranno usati i file di default\n");
			printf("Se si desidera, specificare 2 infile e 2 outfile come parametri\n\n");
			fin1 = fopen("input1.dat", "r");
			fin2 = fopen("input2.dat", "r");
			fout1 = fopen("out1.1.dat", "w");
			fout2 = fopen("out2.2.dat", "w");
			if(fin1==NULL || fin2==NULL || fout1==NULL || fout2==NULL)
			{
				printf("ERRORE nell'apertura dei file\n");
				return -1;
			}
			break;
		}
		case 5:   // l'utente ha specificato dei file: l'elaborazione procede su quelli
		{
			fin1 = fopen(argv[1], "r");
			fin2 = fopen(argv[2], "r");
			fout1 = fopen(argv[3], "w");
			fout2 = fopen(argv[4], "w");
			if(fin1==NULL || fin2==NULL || fout1==NULL || fout2==NULL)
			{
				printf("ERRORE nell'apertura dei file\n");
				return -1;
			}
			break;
		}
	}
	
	p1 = carica_struct(fin1, 0);    // la funzione che legge i file, alloca la memoria e riempie le strutture...
	p2 = carica_struct(fin2, 1);
	if(p1==NULL || p2==NULL)
	{
		printf("ERRORE nel caricamento dei dati\n");   // ... ed i relativi controlli
		return -2;
	}
	if(num[0] != num[1])
	{
		printf("ERRORE: dati non coerenti\n");
		return -3;
	}
	
	scalare(p1, p2);                // le vere e proprie elaborazioni: ogni funzione contiene anche la stampa dei risultati
	
	stampa_estremi(p1, fout1);
	stampa_somme(p1, fout1);
	stampa_estremi(p2, fout2);
	stampa_somme(p2, fout2);
	
	free(p1);
	free(p2);
	fclose(fout1);
	fclose(fout2);
		
	return 0;
}	
	
/* 
   Le clausole di lunghezza dei numeri nelle istruzioni di stampa sono state inserite per un mero fatto estetico dopo l'esame 
   dei file di input. Possono essere comodamente rimosse se necessario.
*/

/* NO FATE BUT WHAT WE MADE */ 
